/*
 * TCS Dependency Map Renderer
 *
 * Copyright (c) 2005 TopCoder, Inc. All rights reserved.
 *
 * Author: Pops, TCSDeveloper
 */

using System;
using System.IO;
using System.Collections;
using NUnit.Framework;

namespace TopCoder.Rules.DependencyMap.View.StressTests
{
	/// <summary>
	/// Stress test class for the DependencyRenderer class.
	/// </summary>
	[TestFixture]
	public class StressRenderer
	{
        /// <summary>
        /// Sample dependency. As per the component specification
        /// </summary>
        Dependency phoneticSpellCheck;

        /// <summary>
        /// Dependency purposely built during setup to have
        /// a large number of sub-elements
        /// </summary>
        Dependency myDependency;

        /// <summary>
        /// number of renderers used for testing
        /// </summary>
        private const int maxNumRenderers = 100;

        /// <summary>
        /// number of levels the test dependency tree will have
        /// </summary>
        private const int maxNumLevels = 4;

        /// <summary>
        /// number of sub-elements for each level of the test dependency
        /// </summary>
        private const int maxNumElements = 10;

        /// <summary>
        /// List of Diagram renderers
        /// </summary>
        private IList liste = new ArrayList();

        /// <summary>
        /// Template name for the test files
        /// </summary>
        private const string testFilePath = @"../../test_files/file{0}";

        /// <summary>
        /// Template name for the dependants
        /// </summary>
        private const string dependantName = "level{0} element{1}";

        /// <summary>
        /// counter for number of dependants
        /// </summary>
        private int totalCount;
        
        /// <summary>
        /// Sets up the dependency and renderer list used to test
        /// </summary>
        [TestFixtureSetUp]
        public void Setup()
        {
            //create the sample dependency graph
            Dependency configManager = new Dependency("Config Manager");
            Dependency spellCheck = new Dependency("Spell Check");
            spellCheck.AddDependant(configManager);
            Dependency phoneticPatternMatching = new Dependency("Phonetic Pattern Matching");
            phoneticSpellCheck = new Dependency("Phonetic Spell Check");
            phoneticSpellCheck.AddDependant(configManager);
            phoneticSpellCheck.AddDependant(spellCheck);
            phoneticSpellCheck.AddDependant(phoneticPatternMatching);

            //creates the renderers and add them to the list
            for(int i = 0; i < maxNumRenderers; ++i)
            {
                string fileName = String.Format(testFilePath, i);
                liste.Add(new SVGRenderer(fileName));
            }
            
            //create the large dependency graph
            totalCount = 1;
            myDependency = new Dependency("largeDependency");
            Queue currentLevel = new Queue();
            currentLevel.Enqueue(myDependency);
            for(int i = 0; i < maxNumLevels; ++i)
            {
                Queue nextLevel = new Queue();
                while(currentLevel.Count > 0)
                {
                    Dependency current = (Dependency)currentLevel.Dequeue();
                    for(int j = 0; j < maxNumElements; ++j)
                    {
                        Dependency d = new Dependency(String.Format(dependantName, i, j));
                        current.AddDependant(d);
                        nextLevel.Enqueue(d);
                        ++totalCount;
                    }
                }
                currentLevel = new Queue(nextLevel);
            }
        }

        /// <summary>
        /// Removes the temporary files
        /// </summary>
        [TestFixtureTearDown]
        public void DeleteTempFiles()
        {
            for(int i = 0; i < maxNumRenderers; ++i)
            {
                string fileName = String.Format(testFilePath, i);
                File.Delete(fileName);
            }
        }

        /// <summary>
        /// Attempts to render a dependency graph
        /// with a large number of renderers
        /// </summary>
        [Test]
        public void TestManyRenderers()
        {
            // Create the Dependency Renderer and add the diagram renderers
            DependencyRenderer renderer = new DependencyRenderer();
            for(int i = 0; i < maxNumRenderers; ++i)
            {
                renderer.AddDiagramRenderer((IDiagramRenderer)liste[i]);
            }
            DateTime startTime = DateTime.Now;
            // Render the dependency
            renderer.Render(phoneticSpellCheck);
            TimeSpan ts = DateTime.Now - startTime;
            Console.WriteLine("Rendering to {0} renderers took {1} ms", maxNumRenderers, ts.TotalMilliseconds);
        }

        /// <summary>
        /// Attempts to render a large dependency graph
        /// </summary>
        [Test]
        public void TestLargeGraph()
        {
            // Create the Dependency Renderer and add a diagram renderer
            DependencyRenderer renderer = new DependencyRenderer();
            renderer.AddDiagramRenderer((IDiagramRenderer)liste[0]);
            DateTime startTime = DateTime.Now;
            // Render the dependency
            renderer.Render(myDependency);
            TimeSpan ts = DateTime.Now - startTime;
            Console.WriteLine("Rendering a graph with {0} levels and {1} elements per level  (total of {2} elements) took {3} ms", maxNumLevels, maxNumElements, totalCount, ts.TotalMilliseconds);
        }
	}
}
